/*************************************************************************
 *  Compilation:  javac Fibonacci.java
 *  Execution:    java Fibonacci N
 *
 *  Computes and prints the first N Fibonacci numbers.
 *
 *  WARNING:  this program is spectacularly inefficient and is meant
 *            to illustrate a performance bug, e.g., set N = 45.
 *
 *
 *   % java Fibonacci 7
 *   1: 1
 *   2: 1
 *   3: 2
 *   4: 3
 *   5: 5
 *   6: 8
 *   7: 13
 *
 *   Remarks
 *   -------
 *    - The 93rd Fibonacci number would overflow a long, but this
 *      will take so long to compute with this function that we
 *      don't bother to check for overflow.
 *
 *************************************************************************/

import java.io.*;
import java.util.Date;

public class FibonacciJavaThread extends Thread {
	int n;

	public FibonacciJavaThread(int n, String name) {
		super(name);
		this.n = n;
	}

	public static long fib(int n) {
		if (n <= 1)
			return n;
		else {
			long res = fib(n - 1) + fib(n - 2);
			return res;
		}
	}

	public void run() {
		fib(n);
	}

	public static void main(String[] args) throws IOException,
			InterruptedException {

		int nFibo;
		int nbThread;

		if (args.length < 2) {
			System.out.println("USAGE : n_fibo nb_threads\n"
					+ "        n_fibo : an integer\n"
					+ "        n_threads : an integer\n");
			return;
		}

		nFibo = Integer.parseInt(args[0]);
		nbThread = Integer.parseInt(args[1]);

		// Début du chrono
		Date debut = new Date();

		FibonacciJavaThread[] tab = new FibonacciJavaThread[nbThread];

		for (int i = 0; i < nbThread; i++) {
			tab[i] = new FibonacciJavaThread(nFibo, "Thread " + (i + 1));
		}
		for (int i = 0; i < nbThread; i++) {
			tab[i].start();
		}
		for (int i = 0; i < nbThread; i++) {
			tab[i].join();
		}

		// Fin du chrono
		Date duree = new Date(new Date().getTime() - debut.getTime());
		double realtime = (duree.getTime() / 1000.0);
		System.out.println(nFibo + ";" + nbThread + ";JavaThread;" + realtime);
	}

}
